package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.asset.BaseInfoDTO;
import cn.fastfun.controller.dto.asset.RentInfoDTO;
import cn.fastfun.controller.dto.asset.RentInfoFigureDTO;
import cn.fastfun.controller.param.*;
import cn.fastfun.service.SysExcelFieldService;
import cn.fastfun.service.SysService;
import cn.fastfun.service.entity.AppAssetConf;
import cn.fastfun.service.impl.AppAssetConfServiceImpl;
import cn.fastfun.service.impl.AppAssetProfitServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bridge.dto.*;
import com.bridge.exception.ApiRuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.0 资产收益分析"})
@RestController
@RequestMapping("/api/v1/appassetprofit")
public class ApiAssetProfitController {

    @Autowired
    AppAssetProfitServiceImpl appAssetService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    @Autowired
    AppAssetConfServiceImpl appAssetConfService;



    @ApiOperation(value = "基础信息")
    @RequestMapping(value = "getBaseInfo", method = RequestMethod.POST)
    public ApiDTO getBaseInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())){
            throw new ApiRuntimeException("请输入sn");
        }
        BaseInfoDTO baseInfo = appAssetService.getBaseInfo(param);
        return ApiDTO.ok("成功", baseInfo);
    }

    @ApiOperation(value = "其他信息列表")
    @RequestMapping(value = "getProfitInfo", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfo(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())){
            throw new ApiRuntimeException("请输入sn");
        }
        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(false);
            Object rentInfoDTO = appAssetService.getRentInfo(param);
            response = (RentInfoDTO) rentInfoDTO;
        }
        // 运维信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(2)) {
            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOmTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOmFaultCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOmPlatformCost(new BigDecimal(res.get("total").toString()));

            response = (RentInfoDTO) rentInfoDTO;
        }

        // 赔偿信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(3)) {
            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setComTotalCost(new BigDecimal(res.get("total").toString()));
            response = (RentInfoDTO) rentInfoDTO;
        }

        // 运营信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(5)) {
            BaseInfoDTO baseInfo = (BaseInfoDTO) getBaseInfo(param).getData();
            RentInfoDTO rentInfoDTO = new RentInfoDTO();

            if (baseInfo != null) {
                if (baseInfo.getStatus().equals(0) || baseInfo.getStatus().equals(14))
                    rentInfoDTO.setOpStorageState("未出库");
                else
                    rentInfoDTO.setOpStorageState("未出库");
            }

            param.setTableOrder(1);
            RentInfoDTO assetProfitInfo =(RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetProfitInfo != null){
                rentInfoDTO.setOpCurrentCash(assetProfitInfo.getCash()==null?new BigDecimal(0):assetProfitInfo.getCash());
            }


            param.setTableOrder(5);

            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOpTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOpStorageCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOpTransportCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 3);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOpSaleCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 4);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOpPlatformCost(new BigDecimal(res.get("total").toString()));
            response = (RentInfoDTO) rentInfoDTO;
        }

        // 金融信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(6)) {
            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setFiPaidCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setFiResidueCost(new BigDecimal(res.get("total").toString()));
            response = (RentInfoDTO) rentInfoDTO;
        }

        // 处置信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(7)) {
            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setHandleProfit(new BigDecimal(res.get("total").toString()));

            response = (RentInfoDTO) rentInfoDTO;
        }

        // 其他信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(8)) {
            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            QueryWrapper<AppAssetConf> queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 1);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            Map<String, Object> res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOtherTotalCost(new BigDecimal(res.get("total").toString()));

            queryWrapper = new QueryWrapper();
            queryWrapper.eq("is_delete", 0);
            queryWrapper.eq("sn", param.getSn());
            queryWrapper.gt("time", param.getTimeStart());
            queryWrapper.lt("time", param.getTimeEnd());
            queryWrapper.eq("table_order", param.getTableOrder());
            queryWrapper.eq("type", 2);
            queryWrapper.select("ifnull(sum(amount),0) as total");
            res = appAssetConfService.getMap(queryWrapper);
            rentInfoDTO.setOtherTotalIncome(new BigDecimal(res.get("total").toString()));

            response = (RentInfoDTO) rentInfoDTO;
        }

        // 收入汇总信息
        else if (param.getTableOrder() != null && param.getTableOrder().equals(9)) {

            RentInfoDTO rentInfoDTO = new RentInfoDTO();
            param.setTableOrder(1);
            param.setFigureOrNot(false);
            RentInfoDTO rentInfo = (RentInfoDTO) appAssetService.getRentInfo(param);
            if (rentInfo != null){
                rentInfoDTO.setNetRental(rentInfo.getNetRental());
                rentInfoDTO.setCashIn(rentInfo.getCashIn());
                rentInfoDTO.setSecurityCash(rentInfo.getSecurityCash());
            }

            param.setTableOrder(7);
            RentInfoDTO assetHandleInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetHandleInfo != null){
                rentInfoDTO.setHandleProfit(assetHandleInfo.getHandleProfit());
            }

            param.setTableOrder(2);
            RentInfoDTO assetOmInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetOmInfo != null){
                rentInfoDTO.setOmTotalCost(assetOmInfo.getOmTotalCost());
            }

            param.setTableOrder(3);
            RentInfoDTO assetComInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetComInfo != null){
                rentInfoDTO.setComTotalCost(assetComInfo.getComTotalCost());
            }

            param.setTableOrder(4);
            RentInfoDTO assetSaleInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetSaleInfo != null){
                rentInfoDTO.setSaleTotalCost(assetSaleInfo.getSaleTotalCost());
            }

            param.setTableOrder(5);
            RentInfoDTO assetOpInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetOpInfo != null){
                rentInfoDTO.setOpTotalCost(assetOpInfo.getOpTotalCost());
            }

            param.setTableOrder(6);
            RentInfoDTO assetFiInfo = (RentInfoDTO) getAssetProfitInfo(param).getData();
            if (assetFiInfo != null){
                rentInfoDTO.setFiPaidCost(assetFiInfo.getFiPaidCost());
            }

            rentInfoDTO.setNetIncome(rentInfoDTO.getNetRental().add(rentInfoDTO.getCashIn()).add(rentInfoDTO.getSecurityCash()).add(rentInfoDTO.getHandleProfit()).add(
                    rentInfoDTO.getOmTotalCost()).add(rentInfoDTO.getComTotalCost()).add(rentInfoDTO.getSaleTotalCost()).add(rentInfoDTO.getOpTotalCost()).add(
                            rentInfoDTO.getFiPaidCost()));
            response = (RentInfoDTO) rentInfoDTO;
        }

        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "其他信息绘图")
    @RequestMapping(value = "getProfitInfoFigure", method = RequestMethod.POST)
    public ApiDTO getAssetProfitInfoFigure(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param) throws Exception {

        if (!StringUtils.hasLength(param.getSn())){
            throw new ApiRuntimeException("请输入sn");
        }
        // 租赁信息
        Object response = null;
        if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
            param.setFigureOrNot(true);
            param.setIndex(0);
            param.setLength(0);
            Object rentInfoFigureDTO = appAssetService.getRentInfo(param);
            response = (RentInfoFigureDTO) rentInfoFigureDTO;
        }

        return ApiDTO.ok("成功", response);
    }

    @ApiOperation(value = "导出")
    @RequestMapping(value = "export", method = RequestMethod.POST)
    public void export(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) AssetProfitParam param, HttpServletResponse response) {

        if (!StringUtils.hasLength(param.getSn())){
            throw new ApiRuntimeException("请输入sn");
        }
        try {
            // 租赁信息

            if (param.getTableOrder() != null && param.getTableOrder().equals(1)) {
                param.setFigureOrNot(false);
                RentInfoDTO rentInfoDTO = (RentInfoDTO) appAssetService.getRentInfo(param);
                List<Map<String, Object>> data = new ArrayList<>();
                rentInfoDTO.getAllInfoList().forEach(p -> {
                    data.add(new HashMap<String, Object>() {{
                        put("time", p.getTime());
                        put("type", p.getType());
                        put("amount", p.getAmount());
                        put("orderNo", p.getOrder());
                        put("remark", p.getDescription());
                    }});
                });
                sysExcelFieldService.outExcelFile(sysExcelFieldService.export("rent_info", data), response, "租赁信息.xlsx");
            }
            if (param.getTableOrder() != null && (param.getTableOrder().equals(2) || param.getTableOrder().equals(3)
                    || param.getTableOrder().equals(5)|| param.getTableOrder().equals(6)
                    || param.getTableOrder().equals(7)|| param.getTableOrder().equals(8))) {
                AssetPageQueryParam appAssetConf = new AssetPageQueryParam();
                appAssetConf.setSn(param.getSn());
                appAssetConf.setTableOrder(param.getTableOrder());
                appAssetConf.setTimeStart(param.getTimeStart());
                appAssetConf.setTimeEnd(param.getTimeEnd());
                appAssetConf.setIndex(param.getIndex());
                appAssetConf.setLength(param.getLength());
                appAssetConf.setType(param.getType());
                Page<AppAssetConf> page = new Page<>(appAssetConf.getIndex(), appAssetConf.getLength());
                QueryWrapper<AppAssetConf> appAssetConfQueryWrapper = new QueryWrapper<>();
                appAssetConfQueryWrapper.eq("is_delete", 0);
                if (StringUtils.hasLength(appAssetConf.getSn())){
                    appAssetConfQueryWrapper.like("sn", appAssetConf.getSn());
                }
                if (appAssetConf.getTableOrder() !=null){
                    appAssetConfQueryWrapper.eq("table_order", appAssetConf.getTableOrder());
                }
                if (appAssetConf.getType() != null){
                    appAssetConfQueryWrapper.eq("type", appAssetConf.getType());
                }
                if (appAssetConf.getTimeStart() != null){
                    appAssetConfQueryWrapper.gt("time", appAssetConf.getTimeStart());
                }
                if (appAssetConf.getTimeEnd() != null){
                    appAssetConfQueryWrapper.lt("time", appAssetConf.getTimeEnd());
                }
                Page<AppAssetConf> appAssetConfPage = appAssetConfService.page(page, appAssetConfQueryWrapper);
                List<Map<String, Object>> data = new ArrayList<>();
                if (appAssetConfPage.getTotal() > 0) {
                    appAssetConfPage.getRecords().forEach(p -> {
                        data.add(new HashMap<String, Object>() {{
                            put("time", p.getTime());
                            put("type", p.getType());
                            put("amount", p.getAmount());
                            put("remark", "");
                        }});
                    });
                }
                switch (param.getTableOrder()){
                    case 2:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("om_info", data), response, "运维信息.xlsx");break;
                    case 3:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("com_info", data), response, "赔偿信息.xlsx");break;
                    case 5:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("op_info", data), response, "运营信息.xlsx");break;
                    case 6:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("fi_info", data), response, "金融成本信息.xlsx");break;
                    case 7:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("handle_info", data), response, "处置信息.xlsx");break;
                    case 8:sysExcelFieldService.outExcelFile(sysExcelFieldService.export("other_info", data), response, "其他信息.xlsx");break;

                    default:break;
                }

            }
        } catch (Exception e) {
            throw new ApiRuntimeException("模板下载失败");

//        return ApiDTO.ok("成功");
        }
    }
}